==> libxfcegui4-4.1.22-cvs.tar.gz: libxfcegui4-4.1.22/libxfcegui4/dialogs.c <==
/* $Id: dialogs.c,v 1.24 2004/03/20 18:31:52 jasper Exp $ */
/*-
 * Copyright (c) 2003 Jasper Huijsmans <huysmans@users.sourceforge.net>
 * Copyright (c) 2003 Benedikt Meurer <benny@xfce.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ctype.h>
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include <stdio.h>
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <glib.h>
#include <gtk/gtk.h>

#include "dialogs.h"

#ifndef PATH_MAX
#define DEFAULT_LENGTH 1024
#else
#if (PATH_MAX < 1024)
#define DEFAULT_LENGTH 1024
#else
#define DEFAULT_LENGTH PATH_MAX
#endif
#endif

/* max. length of a message */
#define MAXMESSAGELEN   2048

extern char **environ;

/* create a header with optional icon (may be NULL) in larger bold font;
 * background and foreground colors are taken from gtk */

static void
private_cb_eventbox_style_set (GtkWidget * widget, GtkStyle * old_style)
{
    static gboolean recursive = 0;
    GtkStyle *style;

    if (recursive > 0)
	return;

    ++recursive;
    style = gtk_widget_get_style (widget);
    gtk_widget_modify_bg (widget, GTK_STATE_NORMAL,
			  &style->bg[GTK_STATE_SELECTED]);
    --recursive;
}

static void
private_cb_label_style_set (GtkWidget * widget, GtkStyle * old_style)
{
    static gboolean recursive = 0;
    GtkStyle *style;

    if (recursive > 0)
	return;

    ++recursive;
    style = gtk_widget_get_style (widget);
    gtk_widget_modify_fg (widget, GTK_STATE_NORMAL,
			  &style->fg[GTK_STATE_SELECTED]);
    --recursive;
}

/* compat stub, remove when the time has come */
GtkWidget*
create_header_with_image (GtkWidget *image, const char *text)
{
        return xfce_create_header_with_image (image, text);
}

/**
 * xfce_create_header_with_image:
 * @image: a GtkImage or NULL if no image should be displayed in the header.
 * @text: the text to be displayed in the header.
 *
 * Creates a header with an optional @image (may be NULL) in larger bold
 * font. Background and foreground colors are taken from Gtk+ style.
 *
 * Return value: the container widget that contains the header widgets.
 **/
GtkWidget*
xfce_create_header_with_image (GtkWidget   *image,
                               const gchar *text)
{
    GtkWidget *eventbox, *label, *hbox;
    GtkStyle *style;
    char *markup;

    eventbox = gtk_event_box_new ();
    gtk_widget_show (eventbox);

    hbox = gtk_hbox_new (FALSE, 12);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
    gtk_widget_show (hbox);
    gtk_container_add (GTK_CONTAINER (eventbox), hbox);

    if (image)
    {
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    }

    style = gtk_widget_get_style (eventbox);
    gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL,
			  &style->bg[GTK_STATE_SELECTED]);

    markup =
	g_strconcat ("<span size=\"larger\" weight=\"bold\">", text,
		     "</span>", NULL);
    label = gtk_label_new (markup);
    g_free (markup);
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_widget_show (label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    
    style = gtk_widget_get_style (label);
    gtk_widget_modify_fg (label, GTK_STATE_NORMAL,
			  &style->fg[GTK_STATE_SELECTED]);

    g_signal_connect_after (G_OBJECT (eventbox), "style_set",
			    G_CALLBACK (private_cb_eventbox_style_set), NULL);
    g_signal_connect_after (G_OBJECT (label), "style_set",
			    G_CALLBACK (private_cb_label_style_set), NULL);

    return eventbox;
}

/* compat stub, remove when the time has come */
GtkWidget*
create_header (GdkPixbuf *icon, const char *text)
{
    return xfce_create_header (icon, text);
}

/**
 * xfce_create_header:
 * @icon: a GdkPixbuf or NULL if no icon should be displayed in the header.
 * @text: a text to be displayed in the header.
 *
 * Creates a header with an optional @icon (may be NULL) in larger bold
 * font. Background and foreground colors are taken from Gtk+ style.
 *
 * Return value: the container widget that contains the header widgets.
 **/
GtkWidget*
xfce_create_header (GdkPixbuf *icon, const gchar *text)
{
    GtkWidget *image = NULL;
    
    if (icon)
	image = gtk_image_new_from_pixbuf (icon);

    return xfce_create_header_with_image (image, text);
}

/* compat stub */
GtkWidget*
mixed_button_new (const char *stock, const char *text)
{
    return xfce_create_mixed_button (stock, text);
}

/**
 * xfce_create_mixed_button:
 * @stock: a stock item name.
 * @text: a text to display.
 *
 * Creates a button with both @stock icon and @text.
 *
 * Return value: the newly created mixed button widget.
 **/
GtkWidget*
xfce_create_mixed_button (const gchar *stock, const gchar *text)
{
    GtkWidget *button, *align, *image, *hbox, *label;

    button = gtk_button_new ();
    label = gtk_label_new_with_mnemonic (text);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);

    image = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_BUTTON);
    hbox = gtk_hbox_new (FALSE, 2);

    align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);

    gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (button), align);
    gtk_container_add (GTK_CONTAINER (align), hbox);
    gtk_widget_show_all (align);

    return button;
}

/* compat stub */
GtkWidget*
small_label (const gchar *text)
{
    return xfce_create_small_label (text);
}

/**
 * xfce_create_small_label:
 * @text: label text.
 *
 * Creates a small italic label using Gtk's markup functionality.
 *
 * Return value: the newly created label.
 **/
GtkWidget*
xfce_create_small_label (const gchar *text)
{
    GtkWidget *label;
    gchar *tmp;

    g_return_val_if_fail (text != NULL, NULL);

    tmp = g_strdup_printf ("<small><i>%s</i></small>", text);
    label = gtk_label_new (tmp);
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    g_free (tmp);

    return label;
}

static void
message_dialog (GtkMessageType type, const gchar * message)
{
    GtkWidget *dlg;

    dlg = gtk_message_dialog_new (NULL,
				  GTK_DIALOG_MODAL,
				  type, GTK_BUTTONS_CLOSE, message);

    gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
    
    gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
    gtk_dialog_run (GTK_DIALOG (dlg));
    gtk_widget_destroy (dlg);
}

/**
 * xfce_err:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal error dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_err (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_verr (format, ap);
    va_end (ap);
}

/**
 * xfce_verr:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal error dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_verr (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_ERROR, message_buffer);
}

/**
 * xfce_warn:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal warning dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_warn (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_vwarn (format, ap);
    va_end (ap);
}

/**
 * xfce_vwarn:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal warning dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_vwarn (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_WARNING, message_buffer);
}

/**
 * xfce_info:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal info dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_info (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_vinfo (format, ap);
    va_end (ap);
}

/**
 * xfce_vinfo:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal info dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_vinfo (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_INFO, message_buffer);
}

/* wrappers around gtk_message_dialog (OBSOLETE) */
void
show_info (const char *text)
{
    message_dialog (GTK_MESSAGE_INFO, text);
}

void
show_warning (const char *text)
{
    message_dialog (GTK_MESSAGE_WARNING, text);
}

void
show_error (const char *text)
{
    message_dialog (GTK_MESSAGE_ERROR, text);
}

/* compat stub */
gboolean
confirm (const char *text, const char *stock, const char *action)
{
    return xfce_confirm (text, stock, action);
}

/**
 * xfce_confirm:
 * @text:     a question text
 * @stock_id: a stock item name
 * @action:   if non-NULL, this text is used on the confirm button together
 *            with the @stock icon.
 *
 * Runs a modal confirmation dialog, that has a 'cancel' and a 'confirm'
 * button. The 'confirm' button text can be set by @action if given.
 *
 * Return value: TRUE if the user confirms, else FALSE.
 **/
gboolean
xfce_confirm (const gchar *text,
              const gchar *stock_id,
              const gchar *action)
{
    GtkWidget *dialog, *button;
    int response = GTK_RESPONSE_NONE;

    dialog = gtk_message_dialog_new (NULL,
				     GTK_DIALOG_MODAL,
				     GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
				     text);

    button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    gtk_widget_show (button);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
				  GTK_RESPONSE_NO);

    if (action)
	button = mixed_button_new (stock_id, action);
    else
	button = gtk_button_new_from_stock (stock_id);
    gtk_widget_show (button);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
				  GTK_RESPONSE_YES);

    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

    response = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_hide (dialog);
    gtk_widget_destroy (dialog);

    if (response == GTK_RESPONSE_YES)
	return TRUE;
    else
	return FALSE;
}

/* command execution with error dialog */

gboolean
exec_command (gchar * command)
{
    gboolean success = TRUE;
    GError *error = NULL;	/* this must be NULL to prevent crash :( */

    g_return_val_if_fail (command != NULL, FALSE);

    if (!g_spawn_command_line_async (command, &error))
    {
	char *msg = g_strcompress (error->message);
	char *text =
	    g_strconcat ("Could not run command: ", command, ":\n", msg,
			 NULL);
	show_error (text);
	g_free (msg);
	g_free (text);
	g_error_free (error);
	success = FALSE;
    }
    return success;
}

gboolean
exec_command_full_with_envp (gchar ** argv, gchar ** envp)
{
    gboolean success = TRUE;
    gboolean retval;
    GError *error = NULL;	/* this must be NULL to prevent crash :( */

    g_return_val_if_fail (argv != NULL, FALSE);

    if (envp == NULL)
    {
	envp = environ;
    }

    retval =
	g_spawn_async (NULL, argv, envp, G_SPAWN_SEARCH_PATH, NULL, NULL,
		       NULL, &error);
    if (!retval)
    {
	gchar *msg = g_strcompress (error->message);
	char *text =
	    g_strconcat ("Could not run command: ", argv[0], ":\n", msg,
			 NULL);
	g_error_free (error);
	g_free (msg);
	show_error (text);
	g_free (text);
	success = FALSE;
    }

    return success;
}

/* vi:set ts=8 sw=4: */

==> libxfcegui4-4.1.24-cvs.tar.gz: libxfcegui4-4.1.24/libxfcegui4/dialogs.c <==
/* $Id: dialogs.c,v 1.26 2004/07/19 02:21:41 kelnos Exp $ */
/*-
 * Copyright (c) 2003 Jasper Huijsmans <huysmans@users.sourceforge.net>
 * Copyright (c) 2003 Benedikt Meurer <benny@xfce.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ctype.h>
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include <stdio.h>
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <glib.h>
#include <gtk/gtk.h>

#include "dialogs.h"

#ifndef PATH_MAX
#define DEFAULT_LENGTH 1024
#else
#if (PATH_MAX < 1024)
#define DEFAULT_LENGTH 1024
#else
#define DEFAULT_LENGTH PATH_MAX
#endif
#endif

/* max. length of a message */
#define MAXMESSAGELEN   2048

extern char **environ;

/* create a header with optional icon (may be NULL) in larger bold font;
 * background and foreground colors are taken from gtk */

static void
private_cb_eventbox_style_set (GtkWidget * widget, GtkStyle * old_style)
{
    static gboolean recursive = 0;
    GtkStyle *style;

    if (recursive > 0)
	return;

    ++recursive;
    style = gtk_widget_get_style (widget);
    gtk_widget_modify_bg (widget, GTK_STATE_NORMAL,
			  &style->bg[GTK_STATE_SELECTED]);
    --recursive;
}

static void
private_cb_label_style_set (GtkWidget * widget, GtkStyle * old_style)
{
    static gboolean recursive = 0;
    GtkStyle *style;

    if (recursive > 0)
	return;

    ++recursive;
    style = gtk_widget_get_style (widget);
    gtk_widget_modify_fg (widget, GTK_STATE_NORMAL,
			  &style->fg[GTK_STATE_SELECTED]);
    --recursive;
}

/* compat stub, remove when the time has come */
GtkWidget*
create_header_with_image (GtkWidget *image, const char *text)
{
        return xfce_create_header_with_image (image, text);
}

/**
 * xfce_create_header_with_image:
 * @image: a GtkImage or NULL if no image should be displayed in the header.
 * @text: the text to be displayed in the header.
 *
 * Creates a header with an optional @image (may be NULL) in larger bold
 * font. Background and foreground colors are taken from Gtk+ style.
 *
 * Return value: the container widget that contains the header widgets.
 **/
GtkWidget*
xfce_create_header_with_image (GtkWidget   *image,
                               const gchar *text)
{
    GtkWidget *eventbox, *label, *hbox;
    GtkStyle *style;
    char *markup;

    eventbox = gtk_event_box_new ();
    gtk_widget_show (eventbox);

    hbox = gtk_hbox_new (FALSE, 12);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
    gtk_widget_show (hbox);
    gtk_container_add (GTK_CONTAINER (eventbox), hbox);

    if (image)
    {
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    }

    style = gtk_widget_get_style (eventbox);
    gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL,
			  &style->bg[GTK_STATE_SELECTED]);

    markup =
	g_strconcat ("<span size=\"larger\" weight=\"bold\">", text,
		     "</span>", NULL);
    label = gtk_label_new (markup);
    g_free (markup);
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_widget_show (label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    
    style = gtk_widget_get_style (label);
    gtk_widget_modify_fg (label, GTK_STATE_NORMAL,
			  &style->fg[GTK_STATE_SELECTED]);

    g_signal_connect_after (G_OBJECT (eventbox), "style_set",
			    G_CALLBACK (private_cb_eventbox_style_set), NULL);
    g_signal_connect_after (G_OBJECT (label), "style_set",
			    G_CALLBACK (private_cb_label_style_set), NULL);

    return eventbox;
}

/* compat stub, remove when the time has come */
GtkWidget*
create_header (GdkPixbuf *icon, const char *text)
{
    return xfce_create_header (icon, text);
}

/**
 * xfce_create_header:
 * @icon: a GdkPixbuf or NULL if no icon should be displayed in the header.
 * @text: a text to be displayed in the header.
 *
 * Creates a header with an optional @icon (may be NULL) in larger bold
 * font. Background and foreground colors are taken from Gtk+ style.
 *
 * Return value: the container widget that contains the header widgets.
 **/
GtkWidget*
xfce_create_header (GdkPixbuf *icon, const gchar *text)
{
    GtkWidget *image = NULL;
    
    if (icon)
	image = gtk_image_new_from_pixbuf (icon);

    return xfce_create_header_with_image (image, text);
}

/* compat stub */
GtkWidget*
mixed_button_new (const char *stock, const char *text)
{
    return xfce_create_mixed_button (stock, text);
}

/**
 * xfce_create_mixed_button:
 * @stock: a stock item name.
 * @text: a text to display.
 *
 * Creates a button with both @stock icon and @text.
 *
 * Return value: the newly created mixed button widget.
 **/
GtkWidget*
xfce_create_mixed_button (const gchar *stock, const gchar *text)
{
    GtkWidget *button, *align, *image, *hbox, *label;

    button = gtk_button_new ();
    label = gtk_label_new_with_mnemonic (text);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);

    image = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_BUTTON);
    hbox = gtk_hbox_new (FALSE, 2);

    align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);

    gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (button), align);
    gtk_container_add (GTK_CONTAINER (align), hbox);
    gtk_widget_show_all (align);

    return button;
}

/* compat stub */
GtkWidget*
small_label (const gchar *text)
{
    return xfce_create_small_label (text);
}

/**
 * xfce_create_small_label:
 * @text: label text.
 *
 * Creates a small italic label using Gtk's markup functionality.
 *
 * Return value: the newly created label.
 **/
GtkWidget*
xfce_create_small_label (const gchar *text)
{
    GtkWidget *label;
    gchar *tmp;

    g_return_val_if_fail (text != NULL, NULL);

    tmp = g_strdup_printf ("<small><i>%s</i></small>", text);
    label = gtk_label_new (tmp);
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    g_free (tmp);

    return label;
}

static void
message_dialog (GtkMessageType type, const gchar * message)
{
    GtkWidget *dlg;

    dlg = gtk_message_dialog_new (NULL,
				  GTK_DIALOG_MODAL,
				  type, GTK_BUTTONS_CLOSE, message);

    gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
    gtk_dialog_run (GTK_DIALOG (dlg));
    gtk_widget_destroy (dlg);
}

/**
 * xfce_err:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal error dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_err (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_verr (format, ap);
    va_end (ap);
}

/**
 * xfce_verr:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal error dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_verr (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_ERROR, message_buffer);
}

/**
 * xfce_warn:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal warning dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_warn (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_vwarn (format, ap);
    va_end (ap);
}

/**
 * xfce_vwarn:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal warning dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_vwarn (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_WARNING, message_buffer);
}

/**
 * xfce_info:
 * @format: printf-style format string.
 * @Varargs: argument list.
 * 
 * Displays a modal info dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_info (const gchar * format, ...)
{
    va_list ap;

    va_start (ap, format);
    xfce_vinfo (format, ap);
    va_end (ap);
}

/**
 * xfce_vinfo:
 * @format: printf-style format string.
 * @ap: variable argument list pointer.
 *
 * Displays a modal info dialog to the user and blocks until the users
 * clicks the close button.
 */
void
xfce_vinfo (const gchar * format, va_list ap)
{
    gchar message_buffer[MAXMESSAGELEN];

    g_return_if_fail (format != NULL);

    g_vsnprintf (message_buffer, sizeof (message_buffer), format, ap);
    message_dialog (GTK_MESSAGE_INFO, message_buffer);
}

/* wrappers around gtk_message_dialog (OBSOLETE) */
void
show_info (const char *text)
{
    message_dialog (GTK_MESSAGE_INFO, text);
}

void
show_warning (const char *text)
{
    message_dialog (GTK_MESSAGE_WARNING, text);
}

void
show_error (const char *text)
{
    message_dialog (GTK_MESSAGE_ERROR, text);
}

/* compat stub */
gboolean
confirm (const char *text, const char *stock, const char *action)
{
    return xfce_confirm (text, stock, action);
}

/**
 * xfce_confirm:
 * @text:     a question text
 * @stock_id: a stock item name
 * @action:   if non-NULL, this text is used on the confirm button together
 *            with the @stock icon.
 *
 * Runs a modal confirmation dialog, that has a 'cancel' and a 'confirm'
 * button. The 'confirm' button text can be set by @action if given.
 *
 * If @stock_id is equal to GTK_STOCK_YES, the 'cancel' button becomes a 'no' button.
 *
 * Return value: TRUE if the user confirms, else FALSE.
 **/
gboolean
xfce_confirm (const gchar *text,
              const gchar *stock_id,
              const gchar *action)
{
    GtkWidget *dialog, *button;
    int response = GTK_RESPONSE_NONE;

    dialog = gtk_message_dialog_new (NULL,
				     GTK_DIALOG_MODAL,
				     GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
				     text);
    if (strcmp (stock_id, GTK_STOCK_YES) == 0)
        button = gtk_button_new_from_stock (GTK_STOCK_NO);
    else
        button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);

    gtk_widget_show (button);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
				  GTK_RESPONSE_NO);

    if (action)
	button = mixed_button_new (stock_id, action);
    else
	button = gtk_button_new_from_stock (stock_id);
    gtk_widget_show (button);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
				  GTK_RESPONSE_YES);

    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

    response = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_hide (dialog);
    gtk_widget_destroy (dialog);

    if (response == GTK_RESPONSE_YES)
	return TRUE;
    else
	return FALSE;
}

/* command execution with error dialog */

gboolean
exec_command (gchar * command)
{
    gboolean success = TRUE;
    GError *error = NULL;	/* this must be NULL to prevent crash :( */

    g_return_val_if_fail (command != NULL, FALSE);

    if (!g_spawn_command_line_async (command, &error))
    {
	char *msg = g_strcompress (error->message);
	char *text =
	    g_strconcat ("Could not run command: ", command, ":\n", msg,
			 NULL);
	show_error (text);
	g_free (msg);
	g_free (text);
	g_error_free (error);
	success = FALSE;
    }
    return success;
}

gboolean
exec_command_full_with_envp (gchar ** argv, gchar ** envp)
{
    gboolean success = TRUE;
    gboolean retval;
    GError *error = NULL;	/* this must be NULL to prevent crash :( */

    g_return_val_if_fail (argv != NULL, FALSE);

    if (envp == NULL)
    {
	envp = environ;
    }

    retval =
	g_spawn_async (NULL, argv, envp, G_SPAWN_SEARCH_PATH, NULL, NULL,
		       NULL, &error);
    if (!retval)
    {
	gchar *msg = g_strcompress (error->message);
	char *text =
	    g_strconcat ("Could not run command: ", argv[0], ":\n", msg,
			 NULL);
	g_error_free (error);
	g_free (msg);
	show_error (text);
	g_free (text);
	success = FALSE;
    }

    return success;
}

/* vi:set ts=8 sw=4: */
